import os
from google.cloud import secretmanager
import google.auth

class GcpSecretManager:
    """
    Instantiate a GCP Secret Manager object.

    Args:
        credential_file (str): Credential json file
        proxy (str): Proxy address
    """
    def __init__(self, credential_file, proxy=''):
        self.credential_file = credential_file
        self.proxy = proxy
        os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credential_file
        if proxy != '':
            os.environ['HTTP_PROXY'] = proxy
            os.environ['HTTPS_PROXY'] = proxy
        self.credentials, self.project = google.auth.load_credentials_from_file(credential_file)

    def list_secret(self, project_id):
        """
        List all secrets in a project.

        Args:
            project_id (str): Project ID

        Returns:
            secret_list (list): List of secrets
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        parent = f"projects/{project_id}"
        secrets = client.list_secrets(request={"parent": parent})
        results = []
        secret_list = []
        for secret in secrets:
            secret_list.append(secret.name)
        return secret_list
    
    def get_secret(self, project_id, secret_id):
        """
        Get a secret from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID

        Returns:
            result (str): Secret value
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/latest"
        response = client.access_secret_version(request={"name": name})
        result = response.payload.data.decode("UTF-8")
        return result
    
    def get_secret_version(self, project_id, secret_id, secret_version):
        """
        Get a secret from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version

        Returns:
            result (str): Secret value
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        response = client.access_secret_version(request={"name": name})
        result = response.payload.data.decode("UTF-8")
        return result
    
    def get_secret_versions(self, project_id, secret_id):
        """
        Get all secret versions from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID

        Returns:
            secret_list (list): Secret version list
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}"
        response = client.list_secret_versions(request={"parent": name})
        result = []
        version_list = []
        for version in response:
            version_list.append(version.name)
        return version_list
    
    def get_secret_version_create_time(self, project_id, secret_id, secret_version):
        """
        Get secret version create time from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version

        Returns:
            result (str): Secret version create time
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        response = client.access_secret_version(request={"name": name})
        result = response.create_time
        return result
    
    def get_secret_version_state(self, project_id, secret_id, secret_version):
        """
        Get secret version state from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version

        Returns:
            result (str): Secret version state
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        response = client.access_secret_version(request={"name": name})
        result = response.state
        return result
    
    def get_secret_version_destroy_time(self, project_id, secret_id, secret_version):
        """
        Get secret version destroy time from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version

        Returns:
            result (str): Secret version destroy time
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        response = client.access_secret_version(request={"name": name})
        result = response.destroy_time
        return result
    
    def create_secret(self, project_id, secret_id):
        """
        Create a secret in a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID

        Returns:
            result (str): Secret name
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        parent = f"projects/{project_id}"
        secret = {"replication": {"automatic": {}}}
        response = client.create_secret(request={"parent": parent, "secret_id": secret_id, "secret": secret})
        result = response.name
        return result
    
    def add_secret_version(self, project_id, secret_id, payload):
        """
        Add a secret version in a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            payload (str): Secret value

        Returns:
            result (str): Secret version name
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        parent = f"projects/{project_id}/secrets/{secret_id}"
        response = client.add_secret_version(request={"parent": parent, "payload": {"data": payload.encode("UTF-8")}})
        result = response.name
        return result
    
    def delete_secret(self, project_id, secret_id):
        """
        Delete a secret from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID

        Returns:
            result (str): It returns 'OK' when successful
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}"
        client.delete_secret(request={"name": name})
        result = 'OK'
        return result
    
    def destroy_secret_version(self, project_id, secret_id, secret_version):
        """
        Destroy a secret version from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version

        Returns:
            result (str): It returns 'OK' when successful
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        client.destroy_secret_version(request={"name": name})
        result = 'OK'
        return result
    
    def disable_secret_version(self, project_id, secret_id, secret_version):
        """
        Disable a secret version from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version

        Returns:
            result (str): It returns 'OK' when successful
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        client.disable_secret_version(request={"name": name})
        result = 'OK'
        return result
    
    def enable_secret_version(self, project_id, secret_id, secret_version):
        """
        Enable a secret version from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version

        Returns:
            result (str): It returns 'OK' when successful
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        client.enable_secret_version(request={"name": name})
        result = 'OK'
        return result
    
    def get_secret_policy(self, project_id, secret_id):
        """
        Get a secret policy from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID

        Returns:
            result (str): Secret policy
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}"
        response = client.get_iam_policy(request={"resource": name})
        result = response
        return result

    def set_secret_policy(self, project_id, secret_id, member, role):
        """
        Set a secret policy from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            member (str): Member
            role (str): Role

        Returns:
            result (str): It returns 'OK' when successful
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}"
        policy = {"bindings": [{"role": role, "members": [member]}]}
        response = client.set_iam_policy(request={"resource": name, "policy": policy})
        result = 'OK'
        return result
    
    def test_iam_permissions(self, project_id, secret_id, permissions):
        """
        Test a secret policy from a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            permissions (list): Permissions

        Returns:
            result (str): It returns 'OK' when successful
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}"
        response = client.test_iam_permissions(request={"resource": name, "permissions": permissions})
        result = response
        return result
    
    def add_secret_version_from_file(self, project_id, secret_id, file_path):
        """
        Add a secret version from a file in a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            file_path (str): File path

        Returns:
            result (str): Secret version name
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        parent = f"projects/{project_id}/secrets/{secret_id}"
        with open(file_path, "rb") as f:
            payload = {"data": f.read()}
        response = client.add_secret_version(request={"parent": parent, "payload": payload})
        result = response.name
        return result
    
    def add_secret_version_from_string(self, project_id, secret_id, payload):
        """
        Add a secret version from a string in a project.

        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            payload (str): Secret value

        Returns:
            result (str): Secret version name
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        parent = f"projects/{project_id}/secrets/{secret_id}"
        response = client.add_secret_version(request={"parent": parent, "payload": {"data": payload.encode("UTF-8")}})
        result = response.name
        return result
    
    def get_secret_version_to_file(self, project_id, secret_id, secret_version, file_path):
        """
        Get a secret version in a project and store it in a file.
        
        Args:
            project_id (str): Project ID
            secret_id (str): Secret ID
            secret_version (str): Secret version
            file_path (str): File path
            
        Returns:
            result (str): It returns 'OK' when successful
        """
        client = secretmanager.SecretManagerServiceClient(credentials=self.credentials)
        name = f"projects/{project_id}/secrets/{secret_id}/versions/{secret_version}"
        response = client.access_secret_version(request={"name": name})
        with open(file_path, "wb") as f:
            f.write(response.payload.data)
        result = 'OK'
        return result
    
    



import paramiko
from paramiko import SSHClient
from scp import SCPClient
class SshConnector:
    """Instantiate an SSH connector.
    Args:
        host (str): SSH host
        port (str): SSH port
        private_key (str): SSH private key file path
        username (str): SSH user
        disable_rsa_512_256 (boolean): If the value is True, then rsa-sha2-512 and rsa-sha2-256 algorithm will be disabled
    """    
    def __init__(self, host, port, private_key, username, disable_rsa_512_256=False):
        self.host = host
        self.port = port
        self.private_key = private_key
        self.username = username
        self.disable_rsa_512_256 = disable_rsa_512_256
        
        self.ssh = SSHClient()
        self.ssh.load_system_host_keys()
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.key = paramiko.RSAKey.from_private_key_file(self.private_key)
        if self.disable_rsa_512_256 == False:
            self.ssh.connect(self.host, port=self.port, username=self.username, pkey=self.key, timeout=3600)
        else:
            self.ssh.connect(self.host, port=self.port, username=self.username, pkey=self.key, timeout=3600, disabled_algorithms=dict(pubkeys=["rsa-sha2-512", "rsa-sha2-256"]))
        
    def scp_upload(self, src, dst):
        """
        Upload a file with SCP
        Args:
            src (str): Path of source file
            dst (str): Path of destination file
        Returns:
            result (str): The result of function
        """        
        scp = SCPClient(self.ssh.get_transport())
        scp.put(src, dst)    
        return """{src} has been uploaded!""".format(src=src)

    def scp_download(self, src, dst):
        """
        Download a file with SCP
        Args:
            src (str): Path of source file
            dst (str): Path of destination file
        Returns:
            result (str): The result of function
        """                
        scp = SCPClient(self.ssh.get_transport())
        scp.get(src, dst)    
        return """{src} has been downloaded!""".format(src=src)


import os
from google.cloud import bigquery
from google.cloud import storage
import pandas as pd

class GcpConnector:
    """
    Instantiate a GCP connector.

    Args:
        credential_file (str): Credential json file
        proxy (str): Proxy address
    """
    def __init__(self, credential_file, proxy=''):
        self.credential_file = credential_file
        self.proxy = proxy
        os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credential_file
        if proxy != '':
            os.environ['HTTP_PROXY'] = proxy
            os.environ['HTTPS_PROXY'] = proxy

    def bq_query_to_dataframe(self, project_id, query, timeout=3600, method=1):
        """
        Submit query to BigQuery and store result into pandas dataframe.

        Args:
            project_id (str): Project ID
            query (str): SQL query
            timeout (int): Query timeout in seconds
            method (int): API that will be used to query (1: google-cloud-bigquery, 2: pandas-gbq)

        Returns:
            result (dataframe)): Result in pandas dataframe
        """
        df = pd.DataFrame()
        bqclient = bigquery.Client(project=project_id)
        query_job = bqclient.query(query)
        if method == 2:
            df = pd.read_gbq(query=query, project_id=project_id)
        else:
            rows = list(query_job.result(timeout=timeout))
            df = pd.DataFrame(data=[list(x.values()) for x in rows], columns=list(rows[0].keys()))
        return df

    def bq_query_non_dql(self, project_id, query):
        """
        Submit non Data Query Language (DQL) type of query to BigQuery. Example: CREATE, DROP, TRUNCATE, INSERT, UPDATE, DELETE.

        Args:
            project_id (str): Project ID
            query (str): SQL query

        Returns:
            result (result): Iterator of row data. Reference: https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.job.QueryJob.html?highlight=job%20result#google.cloud.bigquery.job.QueryJob.result
        """
        bqclient = bigquery.Client(project=project_id)
        query_job = bqclient.query(query)
        results = query_job.result()
        return results

    def bq_export_table_to_gcs(self, project_id, dataset_id, table_id, gcs_uri, format='CSV', delimiter=',', enable_compression=True, compression='GZIP', overwrite=True, region='northamerica-northeast1'):
        """
        Export BigQuery table into Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            table_id (str): Table ID
            dataset_id (str): Dataset ID
            gcs_uri (str): GCS URI as destination. Example: 'gs://my-bucket/my-dir/tickets-20220101-*.csv.gz'
            format (str): File format (CSV, JSON, Avro, Parquet). Default: 'CSV'
            delimiter (str): CSV delimiter character. Default: ','
            enable_compression (boolean): Files will be compressed if the value is True. Default: True
            compression (str): Compression format. Default: GZIP. Reference: https://cloud.google.com/bigquery/docs/exporting-data#export_formats_and_compression_types
            overwrite (boolean): GCS URI destination will be overwritten if the value is True. Default: True
            region (str): Region to run the process. Default: 'northamerica-northeast1'

        Returns:
            result (result): Iterator of row data. Reference: https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.job.QueryJob.html?highlight=job%20result#google.cloud.bigquery.job.QueryJob.result
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        table_ref = dataset_ref.table(table_id)
        job_config = bigquery.job.ExtractJobConfig()
        if enable_compression == True:
            if compression == 'DEFLATE':
                job_config.compression = bigquery.Compression.DEFLATE
            if compression == 'SNAPPY':
                job_config.compression = bigquery.Compression.SNAPPY
            else:
                job_config.compression = bigquery.Compression.GZIP

        extract_job = bqclient.extract_table(table_ref, gcs_uri, location=region, job_config=job_config)
        results = extract_job.result()
        return results

    def gcs_download_single_file(self, project_id, bucket_id, source_blob_path, destination_path):
        """
        Download a single object from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            source_blob_path (str): The path of source object. Example: 'gcs-directory/my-filename.txt'
            destination_path (str): Local destination path. Example: '/my-directory/my-filename.txt'

        Returns:
            result (result): Iterator of row data. Reference: https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.job.QueryJob.html?highlight=job%20result#google.cloud.bigquery.job.QueryJob.result
        """
        gcs_client = storage.Client(project=project_id)
        bucket = gcs_client.bucket(bucket_id)
        blob = bucket.blob(source_blob_path)
        blob.download_to_filename(destination_path)
        results = 'OK'
        return results

    def gcs_download_objects_with_pattern(self, project_id, bucket_id, blob_prefix, destination_dir_path, printout=True):
        """
        Download multiple objects which have same prefix pattern from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            blob_prefix (str): The blob prefix pattern that wil be downloaded. Example: 'gcs-directory/tickets-20220101-'
            destination_dir_path (str): Local destination directory path. Example: '/my-directory'
            printout (boolean): File name will be displayed if this value is true. Default: True
        """

        delimiter='/'
        storage_client = storage.Client(project_id)
        bucket=storage_client.get_bucket(bucket_id)
        # List blobs iterate in folder 
        blobs=bucket.list_blobs(prefix=blob_prefix, delimiter=delimiter) # Excluding folder inside bucket
        for blob in blobs:
            if printout == True:
                print(blob.name)
            filename_raw = blob.name.split('/')
            filename = filename_raw[len(filename_raw)-1]
            destination_uri = '{}{}'.format(destination_dir_path, filename) 
            blob.download_to_filename(destination_uri)
        results = 'OK'   
        return results

    def gcs_upload_single_file(self, project_id, bucket_id, local_file, destination_blob):
        """
        Upload a single object from Google Cloud Storage (GCS).
        
        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            local_file (str): Local file as source. Example: '/local-directory/my-filename.txt'
            destination_blob (str): Destination blob in GCS bucket. Example: 'gcs-directory/my-filename.txt'
        
        Returns:
            result (str): It returns 'OK' when successful
        """
        gcs_client = storage.Client(project=project_id)
        bucket = gcs_client.bucket(bucket_id)
        blob = bucket.blob(destination_blob)
        blob.upload_from_filename(local_file)
        results = 'OK'
        return results            



import requests
import json

class Notification:
    """
    Instantiate a Notification object.

    Args:
        proxy (str): Proxy address
    """    
    def __init__(self, proxy=''):
        self.proxies = {}
        if proxy != '':
            proxies = {
                'http': proxy,
                'https': proxy,
            }

    
    # Sending message through webhook (Google Chat, Slack, etc)
    def send_google_chat_message(self, webhook_url, message):
        """
        Send message through webhook (Google Chat, Slack, etc).

        Args:
            webhook_url (str): Webhook URL
            message (str): Message to be sent

        Returns:
            result (str): Value is 'OK' when successful
        """
        headers = {'Content-type': 'application/json'}
        data = {'text': message}
        response = requests.post(webhook_url, headers=headers, data=json.dumps(data), proxies=self.proxies)
        return 'OK'
    
    
    # Creating Google Chat card as described in https://developers.google.com/chat/how-tos/cards-onclick
    def create_google_chat_card(self, title, subtitle, image_url, image_alt_text, text, button_text, button_url):
        """
        Create Google Chat card.

        Args:
            title (str): Card title
            subtitle (str): Card subtitle
            image_url (str): Image URL
            image_alt_text (str): Image alt text
            text (str): Card text
            button_text (str): Button text
            button_url (str): Button URL

        Returns:
            card (dict): Card to be sent
        """
        card = {
            "cards": [
                {
                    "header": {
                        "title": title,
                        "subtitle": subtitle,
                        "imageUrl": image_url,
                        "imageStyle": "IMAGE"
                    },
                    "sections": [
                        {
                            "widgets": [
                                {
                                    "textParagraph": {
                                        "text": text
                                    }
                                },
                                {
                                    "buttons": [
                                        {
                                            "textButton": {
                                                "text": button_text,
                                                "onClick": {
                                                    "openLink": {
                                                        "url": button_url
                                                    }
                                                }
                                            }
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
        return card

    # Sending card to Google Chat 
    def send_google_chat_card(self, webhook_url, card):
        """
        Send card to Google Chat.

        Args:
            webhook_url (str): Webhook URL
            card (dict): Card to be sent

        Returns:
            result (str): Value is 'OK' when successful
        """
        headers = {'Content-type': 'application/json'}
        data = card
        response = requests.post(webhook_url, headers=headers, data=json.dumps(data), proxies=self.proxies)
        return 'OK'

    # Sending email through SMTP
    def send_smtp_email(self, smtp_server, smtp_port, smtp_username, smtp_password, sender, recipient, subject, message):
        """
        Send email through SMTP.

        Args:
            smtp_server (str): SMTP server address
            smtp_port (str): SMTP port
            smtp_username (str): SMTP username
            smtp_password (str): SMTP password
            sender (str): Sender email address
            recipient (str): Recipient email address
            subject (str): Email subject
            message (str): Email message

        Returns:
            result (str): Value is 'OK' when successful
        """
        import smtplib
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText
        from email.mime.base import MIMEBase
        from email import encoders
        
        msg = MIMEMultipart()
        msg['From'] = sender
        msg['To'] = recipient
        msg['Subject'] = subject
        msg.attach(MIMEText(message, 'plain'))
        text = msg.as_string()
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(smtp_username, smtp_password)
        server.sendmail(sender, recipient, text)
        server.quit()
        return 'OK'
    
    # Sending email through SMTP with attachment
    def send_smtp_email_with_attachment(self, smtp_server, smtp_port, smtp_username, smtp_password, sender, recipient, subject, message, attachment_path):
        """
        Send email through SMTP with attachment.

        Args:
            smtp_server (str): SMTP server address
            smtp_port (str): SMTP port
            smtp_username (str): SMTP username
            smtp_password (str): SMTP password
            sender (str): Sender email address
            recipient (str): Recipient email address
            subject (str): Email subject
            message (str): Email message
            attachment_path (str): Attachment file path

        Returns:
            result (str): Value is 'OK' when successful
        """
        import smtplib
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText
        from email.mime.base import MIMEBase
        from email import encoders
        
        msg = MIMEMultipart()
        msg['From'] = sender
        msg['To'] = recipient
        msg['Subject'] = subject
        msg.attach(MIMEText(message, 'plain'))
        filename = os.path.basename(attachment_path)
        attachment = open(attachment_path, "rb")
        part = MIMEBase('application', 'octet-stream')
        part.set_payload(attachment.read())
        encoders.encode_base64(part)
        part.add_header('Content-Disposition', f"attachment; filename= {filename}")
        msg.attach(part)
        text = msg.as_string()
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(smtp_username, smtp_password)
        server.sendmail(sender, recipient, text)
        server.quit()
        return 'OK'
    
    # Sending email through SMTP in HTML format
    def send_smtp_email_html(self, smtp_server, smtp_port, smtp_username, smtp_password, sender, recipient, subject, message):
        """
        Send email through SMTP in HTML format.

        Args:
            smtp_server (str): SMTP server address
            smtp_port (str): SMTP port
            smtp_username (str): SMTP username
            smtp_password (str): SMTP password
            sender (str): Sender email address
            recipient (str): Recipient email address
            subject (str): Email subject
            message (str): Email message

        Returns:
            result (str): Value is 'OK' when successful
        """
        import smtplib
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText
        from email.mime.base import MIMEBase
        from email import encoders
        
        msg = MIMEMultipart()
        msg['From'] = sender
        msg['To'] = recipient
        msg['Subject'] = subject
        msg.attach(MIMEText(message, 'html'))
        text = msg.as_string()
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(smtp_username, smtp_password)
        server.sendmail(sender, recipient, text)
        server.quit()
        return 'OK'
    
    # Sending email through SMTP in HTML format with attachment
    def send_smtp_email_html_with_attachment(self, smtp_server, smtp_port, smtp_username, smtp_password, sender, recipient, subject, message, attachment_path):
        """
        Send email through SMTP in HTML format with attachment.

        Args:
            smtp_server (str): SMTP server address
            smtp_port (str): SMTP port
            smtp_username (str): SMTP username
            smtp_password (str): SMTP password
            sender (str): Sender email address
            recipient (str): Recipient email address
            subject (str): Email subject
            message (str): Email message
            attachment_path (str): Attachment file path

        Returns:
            result (str): Value is 'OK' when successful
        """
        import smtplib
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText
        from email.mime.base import MIMEBase
        from email import encoders
        
        msg = MIMEMultipart()
        msg['From'] = sender
        msg['To'] = recipient
        msg['Subject'] = subject
        msg.attach(MIMEText(message, 'html'))
        filename = os.path.basename(attachment_path)
        attachment = open(attachment_path, "rb")
        part = MIMEBase('application', 'octet-stream')
        part.set_payload(attachment.read())
        encoders.encode_base64(part)
        part.add_header('Content-Disposition', f"attachment; filename= {filename}")
        msg.attach(part)
        text = msg.as_string()
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(smtp_username, smtp_password)
        server.sendmail(sender, recipient, text)
        server.quit()
        return 'OK'
    




import os
from google.cloud import storage
class GcpCloudStorage:
    """
    Instantiate a GCS connector.
    Args:
        credential_file (str): Credential json file
        proxy (str): Proxy address
    """
    def __init__(self, credential_file, proxy=''):
        self.credential_file = credential_file
        self.proxy = proxy
        os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credential_file
        if proxy != '':
            os.environ['HTTP_PROXY'] = proxy
            os.environ['HTTPS_PROXY'] = proxy

    def download_blob(self, project_id, bucket_id, source_blob_path, destination_path):
        """
        Download a single object from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            source_blob_path (str): The path of source object. Example: 'gcs-directory/my-filename.txt'
            destination_path (str): Local destination path. Example: '/my-directory/my-filename.txt'

        Returns:
            result (result): Iterator of row data. Reference: https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.job.QueryJob.html?highlight=job%20result#google.cloud.bigquery.job.QueryJob.result
        """
        gcs_client = storage.Client(project=project_id)
        bucket = gcs_client.bucket(bucket_id)
        blob = bucket.blob(source_blob_path)
        blob.download_to_filename(destination_path)
        results = 'OK'
        return results    
    
    def download_many_blobs(self, project_id, bucket_id, blob_prefix, destination_dir_path, printout=True):
        """
        Download multiple objects which have same prefix pattern from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            blob_prefix (str): The blob prefix pattern that wil be downloaded. Example: 'gcs-directory/tickets-20220101-'
            destination_dir_path (str): Local destination directory path. Example: '/my-directory'
            printout (boolean): File name will be displayed if this value is true. Default: True
        
        Returns:
            result (str): It returns 'OK' when successful            
        """

        delimiter='/'
        storage_client = storage.Client(project_id)
        bucket=storage_client.get_bucket(bucket_id)
        # List blobs iterate in folder 
        blobs=bucket.list_blobs(prefix=blob_prefix, delimiter=delimiter) # Excluding folder inside bucket
        for blob in blobs:
            if printout == True:
                print(blob.name)
            filename_raw = blob.name.split('/')
            filename = filename_raw[len(filename_raw)-1]
            destination_uri = '{}{}'.format(destination_dir_path, filename) 
            blob.download_to_filename(destination_uri)
        results = 'OK'   
        return results
    
    def upload_blob(self, project_id, bucket_id, local_file, destination_blob):
        """
        Upload a single object from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            local_file (str): Local file as source. Example: '/local-directory/my-filename.txt'
            destination_blob (str): Destination blob in GCS bucket. Example: 'gcs-directory/my-filename.txt'
        
        Returns:
            result (str): It returns 'OK' when successful
        """
        gcs_client = storage.Client(project=project_id)
        bucket = gcs_client.bucket(bucket_id)
        blob = bucket.blob(destination_blob)
        blob.upload_from_filename(local_file)
        results = 'OK'
        return results          
    
    def upload_many_blobs(self, project_id, bucket_id, local_dir, destination_dir):
        """
        Upload multiple objects to Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            local_dir (str): Local directory as source. Example: '/local-directory'
            destination_dir (str): Destination directory in GCS bucket. Example: 'gcs-directory'
        
        Returns:
            result (str): It returns 'OK' when successful
        """
        gcs_client = storage.Client(project=project_id)
        bucket = gcs_client.bucket(bucket_id)
        for filename in os.listdir(local_dir):
            blob = bucket.blob(f'{destination_dir}/{filename}')
            blob.upload_from_filename(f'{local_dir}/{filename}')
        results = 'OK'
        return results

    def delete_blob(self, project_id, bucket_id, blob_name):
        """
        Delete a single object from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            blob_name (str): Blob name. Example: 'gcs-directory/my-filename.txt'
        
        Returns:
            result (str): It returns 'OK' when successful
        """
        gcs_client = storage.Client(project=project_id)
        bucket = gcs_client.bucket(bucket_id)
        blob = bucket.blob(blob_name)
        blob.delete()
        results = 'OK'
        return results
    
    def delete_many_blobs(self, project_id, bucket_id, blob_prefix):
        """
        Delete multiple objects from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            blob_prefix (str): Blob prefix. Example: 'gcs-directory/my-filename.txt'
        
        Returns:
            result (str): It returns 'OK' when successful
        """
        delimiter='/'
        storage_client = storage.Client(project_id)
        bucket=storage_client.get_bucket(bucket_id)
        # List blobs iterate in folder 
        blobs=bucket.list_blobs(prefix=blob_prefix, delimiter=delimiter)
        for blob in blobs:
            blob.delete()
        results = 'OK'
        return results
    
    def list_blobs(self, project_id, bucket_id, blob_prefix):
        """
        List objects from Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            bucket_id (str): Bucket ID
            blob_prefix (str): Blob prefix. Example: 'gcs-directory/my-filename.txt'
        
        Returns:
            blob_list (list): Blob list
        """
        delimiter='/'
        storage_client = storage.Client(project_id)
        bucket=storage_client.get_bucket(bucket_id)
        # List blobs iterate in folder 
        blobs=bucket.list_blobs(prefix=blob_prefix, delimiter=delimiter)
        blob_list = []
        for blob in blobs:
            blob_list.append(blob.name)
        results = 'OK'
        return blob_list
    
    def copy_blob_between_buckets(self, project_id, source_bucket_id, destination_bucket_id, blob_name):
        """
        Copy a single object between buckets in Google Cloud Storage (GCS).

        Args:
            project_id (str): Project ID
            source_bucket_id (str): Source bucket ID
            destination_bucket_id (str): Destination bucket ID
            blob_name (str): Blob name. Example: 'gcs-directory/my-filename.txt'
        
        Returns:
            result (str): It returns 'OK' when successful
        """
        gcs_client = storage.Client(project=project_id)
        source_bucket = gcs_client.bucket(source_bucket_id)
        destination_bucket = gcs_client.bucket(destination_bucket_id)
        blob = source_bucket.blob(blob_name)
        new_blob = source_bucket.copy_blob(blob, destination_bucket, blob_name)
        results = 'OK'
        return results
    
    def copy_many_blobs_between_buckets(self, project_id, source_bucket_id, destination_bucket_id, blob_prefix):
        """
        Copy multiple objects between buckets in Google Cloud Storage (GCS).
        
        Args:
            project_id (str): Project ID
            source_bucket_id (str): Source bucket ID
            destination_bucket_id (str): Destination bucket ID
            blob_prefix (str): Blob prefix. Example: 'gcs-directory/my-filename.txt'
        
        Returns:
            result (str): It returns 'OK' when successful
        """
        delimiter='/'
        storage_client = storage.Client(project_id)
        source_bucket = storage_client.bucket(source_bucket_id)
        destination_bucket = storage_client.bucket(destination_bucket_id)
        # List blobs iterate in folder 
        blobs=source_bucket.list_blobs(prefix=blob_prefix, delimiter=delimiter)
        for blob in blobs:
            new_blob = source_bucket.copy_blob(blob, destination_bucket, blob.name)
        results = 'OK'
        return results


from honeydew.mysql import MysqlConnector
from honeydew.gcp import GcpConnector
from honeydew.gcp_bigquery import GcpBigQuery
from honeydew.gcp_secretmanager import GcpSecretManager
from honeydew.gcp_storage import GcpCloudStorage
from honeydew.gcp_logging import GcpLogging
from honeydew.gcp_translate import GcpTranslate
from honeydew.notification import Notification
from honeydew.ssh import SshConnector
from honeydew.utils import Utils


import calendar
from datetime import datetime
import os
class Utils:
    """Instantiate utilities.
    """    
    def __init__(self, name=''):
        """Instantiate utilities.
        """    
        
    def convert_dt_to_epoch(self, dt):
        """
        Convert datetime in UTC time zone to epoch (unix time)
        Args:
            dt (datetime): datetime
        Returns:
            result (int): epoch or unix time
        """
        return calendar.timegm(dt.utctimetuple())

    def convert_epoch_to_dt(self, epoch):
        """
        Convert epoch to datetime
        Args:
            epoch (int): epoch or unix time
        Returns:
            result (datetime): datetime
        """
        if epoch > 9999999999:
            epoch = round(epoch/1000)
        return datetime.fromtimestamp(s)
    
def compress(dir_path, file_pattern='*', delete_after_compression=False):
    """Compress files with pattern or prefix from a directory. Type "*" for all files.
    Args:
        dir_path (str): Directory path
        file_pattern (str): File pattern or prefix
        delete_after_zip (bool): Delete original files after compression
    """

    import gzip
    import shutil
    from pathlib import Path
    for p in Path(dir_path).glob(file_pattern):
        with open(p, 'rb') as f_in:
            with gzip.open(f"{p}.gz", 'wb') as f_out:
                shutil.copyfileobj(f_in, f_out)
        if delete_after_compression:
            os.remove(p)

def compress_parallel(dir_path, file_pattern='*', delete_after_compressing=False):
    """Compress files with pattern or prefix from a directory parallely. Type "*" for all files."""
    from pathlib import Path  
    from pigz_python import PigzFile
    import os
    for p in Path(dir_path).glob(file_pattern):
        pigz_file = PigzFile(p)
        pigz_file.process_compression_target()
        if delete_after_compressing:
            os.remove(p)            


import mysql.connector as sql
import pandas as pd
from datetime import datetime
from pytz import timezone
import paramiko
from paramiko import SSHClient
from scp import SCPClient
# import logging
class MysqlConnector:
    """Instantiate a DB connector.

    Args:
        host (str): Database host 
        port (str): Database port
        user (str): Username
        password (str): Password
        allow_local_infile (boolean): Local infile is allowed when the value is True

    Returns:
        result (str): Value is 'OK' when successful
    """   
    
    def __init__(self, host, port, user, password, allow_local_infile=False):
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.allow_local_infile = allow_local_infile
        
        self.db_connection = sql.connect(
            host=self.host,
            port=self.port,
            user=self.user,
            password=self.password,
            ssl_disabled=True,
            autocommit=True,
            allow_local_infile=True
        )    
        self.db_cursor = self.db_connection.cursor()
        if allow_local_infile:
            query_str = """SET GLOBAL local_infile=1"""
            self.db_cursor.execute(query_str)
        
    def query_without_fetch(self, query_str):
        """
        Send non DQL query.

        Args:
            query_str (str): sql query

        Returns:
            result (str): Value is 'OK' when successful
        """
        self.db_cursor.execute(query_str)
        return 'OK'
    
    def query_to_dataframe(self, query_str):
        """
        Query and store the result in a dataframe.

        Args:
            query_str (str): sql query

        Returns:
            result (dataframe): Result in a dataframe
        """
        self.db_cursor.execute(query_str)
        table_rows = self.db_cursor.fetchall()
        df = pd.DataFrame(table_rows, columns=self.db_cursor.column_names)
        return df
    
    def load_csv(
        self,
        db_name, 
        table_name, 
        file_name,
        write_disposition,
        delimiter=',', 
        ignore_rows=1,
        is_local_csv=True
    ):
        """
        Load a local CSV file into a table.

        Args:
            db_name (str): Database name where the CSV will be loaded
            table_name (str): Table name where the CSV will be loaded
            file_name (str): CSV file name
            delimiter (str): CSV delimiter character
            ignore_rows (str): Number of rows that will be ignored from the top
            write_disposition (str): Write method to add data into table (WRITE_TRUNCATE, WRITE_APPEND)
            is_local_csv (boolean): If the value is True, then CSV file is in local machine. If the value is False, then CSV file is in remote machine.
            
        Returns:
            result (str): The result of function
        """
        result = ''

        if write_disposition == 'WRITE_TRUNCATE':
            query = 'TRUNCATE TABLE {db_name}.{table_name}'.format(db_name=db_name, table_name=table_name)
            self.db_cursor.execute(query)
            self.db_connection.commit()

        # load table
        if is_local_csv:
            sql_import_table = (""" LOAD DATA LOCAL INFILE '{file_name}' 
                                    INTO TABLE {db_name}.{table_name}
                                    FIELDS TERMINATED BY '{delimiter}' 
                                    LINES TERMINATED BY '\\n'
                                    IGNORE {ignore_rows} ROWS;
            """).format(file_name=file_name, db_name=db_name, table_name=table_name, delimiter=delimiter, ignore_rows=ignore_rows)
        else:
            sql_import_table = (""" LOAD DATA INFILE '{file_name}' 
                                    INTO TABLE {db_name}.{table_name}
                                    FIELDS TERMINATED BY '{delimiter}' 
                                    LINES TERMINATED BY '\\n'
                                    IGNORE {ignore_rows} ROWS;
            """).format(file_name=file_name, db_name=db_name, table_name=table_name, delimiter=delimiter, ignore_rows=ignore_rows)
            
        self.db_cursor.execute(sql_import_table)
        self.db_connection.commit()
        result = 'OK'
        return result


import os
from google.cloud import bigquery
from google.cloud import storage
import pandas as pd

class GcpBigQuery:
    """
    Instantiate a GCP BigQuery connector.

    Args:
        credential_file (str): Credential json file
        proxy (str): Proxy address
    """
    def __init__(self, credential_file, proxy=''):
        self.credential_file = credential_file
        self.proxy = proxy
        os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credential_file
        if proxy != '':
            os.environ['HTTP_PROXY'] = proxy
            os.environ['HTTPS_PROXY'] = proxy

    def query_to_dataframe(self, project_id, query, timeout=3600, method=1):
        """
        Submit query to BigQuery and store result into pandas dataframe

        Args:
            project_id (str): Project ID
            query (str): SQL query
            timeout (int): Query timeout in seconds
            method (int): API that will be used to query (1: google-cloud-bigquery, 2: pandas-gbq)

        Returns:
            result (dataframe)): Result in pandas dataframe
        """
        df = pd.DataFrame()
        bqclient = bigquery.Client(project=project_id)
        query_job = bqclient.query(query)
        if method == 2:
            df = pd.read_gbq(query=query, project_id=project_id)
        else:
            rows = list(query_job.result(timeout=timeout))
            df = pd.DataFrame(data=[list(x.values()) for x in rows], columns=list(rows[0].keys()))
        return df

    def query_non_dql(self, project_id, query):
        """
        Submit non Data Query Language (DQL) type of query to BigQuery. Example: CREATE, DROP, TRUNCATE, INSERT, UPDATE, DELETE

        Args:
            project_id (str): Project ID
            query (str): SQL query

        Returns:
            result (result): Iterator of row data. Reference: https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.job.QueryJob.html?highlight=job%20result#google.cloud.bigquery.job.QueryJob.result
        """
        bqclient = bigquery.Client(project=project_id)
        query_job = bqclient.query(query)
        results = query_job.result()
        return results

    def bq_export_table_to_gcs(self, project_id, dataset_id, table_id, gcs_uri, format='CSV', delimiter=',', enable_compression=True, compression='GZIP', overwrite=True, region='northamerica-northeast1'):
        """
        Export BigQuery table into Google Cloud Storage (GCS)

        Args:
            project_id (str): Project ID
            table_id (str): Table ID
            dataset_id (str): Dataset ID
            gcs_uri (str): GCS URI as destination. Example: 'gs://my-bucket/my-dir/tickets-20220101-*.csv.gz'
            format (str): File format (CSV, JSON, Avro, Parquet). Default: 'CSV'
            delimiter (str): CSV delimiter character. Default: ','
            enable_compression (boolean): Files will be compressed if the value is True. Default: True
            compression (str): Compression format. Default: GZIP. Reference: https://cloud.google.com/bigquery/docs/exporting-data#export_formats_and_compression_types
            overwrite (boolean): GCS URI destination will be overwritten if the value is True. Default: True
            region (str): Region to run the process. Default: 'northamerica-northeast1'

        Returns:
            result (result): Iterator of row data. Reference: https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.job.QueryJob.html?highlight=job%20result#google.cloud.bigquery.job.QueryJob.result
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        table_ref = dataset_ref.table(table_id)
        job_config = bigquery.job.ExtractJobConfig()
        if enable_compression == True:
            if compression == 'DEFLATE':
                job_config.compression = bigquery.Compression.DEFLATE
            if compression == 'SNAPPY':
                job_config.compression = bigquery.Compression.SNAPPY
            else:
                job_config.compression = bigquery.Compression.GZIP

        extract_job = bqclient.extract_table(table_ref, gcs_uri, location=region, job_config=job_config)
        results = extract_job.result()
        return results

    # Load dataframe into BigQuery table
    def bq_load_dataframe(self, project_id, dataset_id, table_id, dataframe, write_disposition='WRITE_TRUNCATE', location='northamerica-northeast1'):
        """
        Load dataframe into BigQuery table

        Args:
            project_id (str): Project ID
            table_id (str): Table ID
            dataset_id (str): Dataset ID
            dataframe (dataframe): Dataframe to be loaded
            write_disposition (str): Write disposition. Default: 'WRITE_TRUNCATE'. Options: 'WRITE_TRUNCATE', 'WRITE_APPEND', 'WRITE_EMPTY'
            location (str): Region to run the process. Default: 'northamerica-northeast1'

        Returns:
            result (result): Iterator of row data. Reference: https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.job.QueryJob.html?highlight=job%20result
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bqclient.dataset(dataset_id)
        table_ref = dataset_ref.table(table_id)
        job_config = bigquery.LoadJobConfig()
        job_config.write_disposition = write_disposition
        job_config.location = location
        job = bqclient.load_table_from_dataframe(dataframe, table_ref, job_config=job_config)
        results = job.result()
        return results
    

    # Get list of tables in a dataset
    def get_table_list(self, project_id, dataset_id):
        """
        Get list of tables in a dataset

        Args:
            project_id (str): Project ID
            dataset_id (str): Dataset ID

        Returns:
            result (list): List of tables in a dataset
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        tables = bqclient.list_tables(dataset_ref)
        table_list = [table.table_id for table in tables]
        return table_list
    
    # Get list of datasets in a project
    def get_dataset_list(self, project_id):
        """
        Get list of datasets in a project

        Args:
            project_id (str): Project ID

        Returns:
            result (list): List of datasets in a project
        """
        bqclient = bigquery.Client(project=project_id)
        datasets = bqclient.list_datasets()
        dataset_list = [dataset.dataset_id for dataset in datasets]
        return dataset_list
    
    # Get list of partitions in a table
    def get_table_properties(self, project_id, dataset_id, table_id):
        """
        Get table properties

        Args:
            project_id (str): Project ID
            dataset_id (str): Dataset ID
            table_id (str): Table ID

        Returns:
            result (list): List of partitions in a table
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        table_ref = dataset_ref.table(table_id)
        table = bqclient.get_table(table_ref)
        return table

    # Get users who have access to a dataset
    def get_dataset_access(self, project_id, dataset_id):
        """
        Get users who have access to a dataset

        Args:
            project_id (str): Project ID
            dataset_id (str): Dataset ID

        Returns:
            result (list): List of users who have access to a dataset
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        dataset = bqclient.get_dataset(dataset_ref)
        access_list = dataset.access_entries
        return access_list
    
    # Get users who have access to a table
    def get_table_access(self, project_id, dataset_id, table_id):
        """
        Get users who have access to a table

        Args:
            project_id (str): Project ID
            dataset_id (str): Dataset ID
            table_id (str): Table ID

        Returns:
            result (list): List of users who have access to a table
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        table_ref = dataset_ref.table(table_id)
        table = bqclient.get_table(table_ref)
        access_list = table.access_entries
        return access_list
    
    # Get table schema
    def get_table_schema(self, project_id, dataset_id, table_id):
        """
        Get table schema

        Args:
            project_id (str): Project ID
            dataset_id (str): Dataset ID
            table_id (str): Table ID

        Returns:
            result (list): List of table schema
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        table_ref = dataset_ref.table(table_id)
        table = bqclient.get_table(table_ref)
        schema = table.schema
        return schema
    
    # Get table description
    def get_table_description(self, project_id, dataset_id, table_id):
        """
        Get table description
        
        Args:
            project_id (str): Project ID
            dataset_id (str): Dataset ID
            table_id (str): Table ID

        Returns:
            result (str): Table description
        """
        bqclient = bigquery.Client(project=project_id)
        dataset_ref = bigquery.DatasetReference(project_id, dataset_id)
        table_ref = dataset_ref.table(table_id)
        table = bqclient.get_table(table_ref)
        description = table.description
        return description
    




import os
from google.cloud import translate_v2 as translate

class GcpTranslate:
    """
    Instantiate a GCP Translate object.

    Args:
        credential_file (str): Credential json file
        proxy (str): Proxy address
    """
    def __init__(self, credential_file, proxy=''):
        self.credential_file = credential_file
        self.proxy = proxy
        os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credential_file
        if proxy != '':
            os.environ['HTTP_PROXY'] = proxy
            os.environ['HTTPS_PROXY'] = proxy

    # Translate text from one language to another language
    def translate_text(self, text, target_language='en'):
        """
        Translate text from one language to another language.

        Args:
            text (str): Text to be translated
            target_language (str): Target language. Default: 'en'

        Returns:
            result (str): Translated text
        """
        translate_client = translate.Client()
        result = translate_client.translate(text, target_language=target_language)
        return result['translatedText']
    


# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# This call to setup() does all the work
setup(
    name="honeydew",
    version='0.1.11',
    description="Collection of connectors for ETL",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://honeydew-lib.github.io/",
    project_urls = {
        'Repository': 'https://github.com/jeffis-ca/honeydew',
        'Documentation': 'https://honeydew-lib.github.io/'
    },
    author="Poltak Jefferson",
    author_email="poltak.jefferson@gmail.com",
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Operating System :: OS Independent"
    ],
    packages=["honeydew"],
    include_package_data=True,
    install_requires=["pandas","numpy","mysql-connector-python","google-cloud","google-auth","google-cloud-bigquery","google-cloud-bigquery-storage","google-cloud-storage","google-cloud-logging","google-cloud-secret-manager","google-cloud-translate", "pandas_gbq", "paramiko", "scp", "pytz", "pigz-python", "clickhouse-connect","mkdocs","mkdocs-material","mkdocs-material-extensions","mkdocstrings","mkdocstrings-python","mkdocs-pymdownx-material-extras","pymdown-extensions","build","twine", "requests"]
)


import os
from google.cloud import logging
import google.auth

class GcpLogging:
    """Instantiate Google Cloud Logging client.
    """
    def __init__(self, credential_file=None, proxy=''):
        """Instantiate Google Cloud Logging client.

        Args:
            credential_file (str): Credential file path. Default: None
            proxy (str): Proxy URL. Default: ''
        """
        if credential_file is not None:
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = credential_file
        if proxy != '':
            os.environ['HTTPS_PROXY'] = proxy
        self.client = logging.Client()
        
    def list_logs(self, project_id, filter_str=None):
        """
        List logs in a project.

        Args:
            project_id (str): Project ID
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'
            
        Returns:
            result (list): List of logs
        """
        logs = self.client.list_logs(project_id=project_id, filter_=filter_str)
        return logs
    
    def list_entries(self, project_id, filter_str=None):
        """
        List log entries in a project.

        Args:
            project_id (str): Project ID
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'

        Returns:
            result (list): List of log entries
        """
        entries = self.client.list_entries(project_id=project_id, filter_=filter_str)
        return entries
    
    def write_log(self, project_id, log_name, resource, labels, text_payload=None, json_payload=None):
        """
        Write a log entry.

        Args:
            project_id (str): Project ID
            log_name (str): Log name. Example: 'my-log'
            resource (dict): Resource. Example: {'type': 'global'}
            labels (dict): Labels. Example: {'env': 'dev', 'app': 'my-app'}
            text_payload (str): Text payload. Example: 'my log message'
            json_payload (dict): JSON payload. Example: {'message': 'my log message'}
            
        Returns:
            result (str): It returns 'OK' when successful
        """
        logger = self.client.logger(log_name)
        logger.log_text(text_payload, resource=resource, labels=labels)
        logger.log_struct(json_payload, resource=resource, labels=labels)
        results = 'OK'
        return results
    
    def delete_log(self, project_id, log_name):
        """
        Delete a log.

        Args:
            project_id (str): Project ID
            log_name (str): Log name. Example: 'my-log'

        Returns:
            result (str): It returns 'OK' when successful
        """
        logger = self.client.logger(log_name)
        logger.delete()
        results = 'OK'
        return results
    
    def delete_many_logs(self, project_id, filter_str=None):
        """
        Delete multiple logs.

        Args:
            project_id (str): Project ID
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'

        Returns:
            result (str): It returns 'OK' when successful
        """
        logs = self.client.list_logs(project_id=project_id, filter_=filter_str)
        for log in logs:
            log.delete()
        results = 'OK'
        return results
    
    def delete_log_entries(self, project_id, filter_str=None):
        """
        Delete log entries.

        Args:
            project_id (str): Project ID
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'

        Returns:
            result (str): It returns 'OK' when successful
        """
        entries = self.client.list_entries(project_id=project_id, filter_=filter_str)
        for entry in entries:
            entry.delete()
        results = 'OK'
        return results
    
    def delete_many_log_entries(self, project_id, filter_str=None):
        """
        Delete multiple log entries.

        Args:
            project_id (str): Project ID
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'

        Returns:
            result (str): It returns 'OK' when successful
        """
        entries = self.client.list_entries(project_id=project_id, filter_=filter_str)
        for entry in entries:
            entry.delete()
        results = 'OK'
        return results
    
    def list_sinks(self, project_id):
        """
        List sinks in a project.

        Args:
            project_id (str): Project ID

        Returns:
            result (list): List of sinks
        """
        sinks = self.client.list_sinks(project_id=project_id)
        return sinks
    
    def create_sink(self, project_id, sink_name, filter_str, destination):
        """
        Create a sink.

        Args:
            project_id (str): Project ID
            sink_name (str): Sink name. Example: 'my-sink'
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'
            destination (dict): Destination. Example: {'writer_identity': 'serviceAccount:

        Returns:
            result (str): It returns 'OK' when successful
        """
        sink = self.client.sink(sink_name, filter_=filter_str, destination=destination)
        sink.create()
        results = 'OK'
        return results
    
    def delete_sink(self, project_id, sink_name):
        """
        Delete a sink.

        Args:
            project_id (str): Project ID
            sink_name (str): Sink name. Example: 'my-sink'

        Returns:
            result (str): It returns 'OK' when successful
        """
        sink = self.client.sink(sink_name)
        sink.delete()
        results = 'OK'
        return results
    
    def delete_many_sinks(self, project_id):
        """
        Delete multiple sinks.

        Args:
            project_id (str): Project ID

        Returns:
            result (str): It returns 'OK' when successful
        """
        sinks = self.client.list_sinks(project_id=project_id)
        for sink in sinks:
            sink.delete()
        results = 'OK'
        return results
    
    def get_sink(self, project_id, sink_name):
        """
        Get a sink.

        Args:
            project_id (str): Project ID
            sink_name (str): Sink name. Example: 'my-sink'

        Returns:
            result (dict): Sink
        """
        sink = self.client.sink(sink_name)
        return sink
    
    def update_sink(self, project_id, sink_name, filter_str, destination):
        """
        Update a sink.

        Args:
            project_id (str): Project ID
            sink_name (str): Sink name. Example: 'my-sink'
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'
            destination (dict): Destination. Example: {'writer_identity': 'serviceAccount:

        Returns:
            result (str): It returns 'OK' when successful
        """
        sink = self.client.sink(sink_name, filter_=filter_str, destination=destination)
        sink.update()
        results = 'OK'
        return results
    
    def list_exclusions(self, project_id):
        """
        List exclusions in a project.

        Args:
            project_id (str): Project ID

        Returns:
            result (list): List of exclusions
        """
        exclusions = self.client.list_exclusions(project_id=project_id)
        return exclusions
    
    def create_exclusion(self, project_id, exclusion_name, filter_str):
        """
        Create an exclusion.

        Args:
            project_id (str): Project ID
            exclusion_name (str): Exclusion name. Example: 'my-exclusion'
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'

        Returns:
            result (str): It returns 'OK' when successful
        """
        exclusion = self.client.exclusion(exclusion_name, filter_=filter_str)
        exclusion.create()
        results = 'OK'
        return results
    
    def delete_exclusion(self, project_id, exclusion_name):
        """
        Delete an exclusion.

        Args:
            project_id (str): Project ID
            exclusion_name (str): Exclusion name. Example: 'my-exclusion'

        Returns:
            result (str): It returns 'OK' when successful
        """
        exclusion = self.client.exclusion(exclusion_name)
        exclusion.delete()
        results = 'OK'
        return results
    
    def delete_many_exclusions(self, project_id):
        """
        Delete multiple exclusions.

        Args:
            project_id (str): Project ID

        Returns:
            result (str): It returns 'OK' when successful
        """
        exclusions = self.client.list_exclusions(project_id=project_id)
        for exclusion in exclusions:
            exclusion.delete()
        results = 'OK'
        return results
    
    def get_exclusion(self, project_id, exclusion_name):
        """
        Get an exclusion.

        Args:
            project_id (str): Project ID
            exclusion_name (str): Exclusion name. Example: 'my-exclusion'

        Returns:
            result (dict): Exclusion
        """
        exclusion = self.client.exclusion(exclusion_name)
        return exclusion
    
    def update_exclusion(self, project_id, exclusion_name, filter_str):
        """
        Update an exclusion.

        Args:
            project_id (str): Project ID
            exclusion_name (str): Exclusion name. Example: 'my-exclusion'
            filter_str (str): Filter string. Example: 'resource.type="bigquery_resource"'

        Returns:
            result (str): It returns 'OK' when successful
        """
        exclusion = self.client.exclusion(exclusion_name, filter_=filter_str)
        exclusion.update()
        results = 'OK'
        return results
    
    def list_monitored_resource_descriptors(self, project_id):
        """
        List monitored resource descriptors in a project.

        Args:
            project_id (str): Project ID

        Returns:
            result (list): List of monitored resource descriptors
        """
        descriptors = self.client.list_monitored_resource_descriptors(project_id=project_id)
        return descriptors
    
    def get_monitored_resource_descriptor(self, project_id, descriptor_name):
        """
        Get a monitored resource descriptor.

        Args:
            project_id (str): Project ID
            descriptor_name (str): Descriptor name. Example: 'my-descriptor'

        Returns:
            result (dict): Monitored resource descriptor
        """
        descriptor = self.client.monitored_resource_descriptor(descriptor_name)
        return descriptor
    
    def list_metric_descriptors(self, project_id):
        """
        List metric descriptors in a project.

        Args:
            project_id (str): Project ID

        Returns:
            result (list): List of metric descriptors
        """
        descriptors = self.client.list_metric_descriptors(project_id=project_id)
        return descriptors
    
    def get_metric_descriptor(self, project_id, descriptor_name):
        """
        Get a metric descriptor.

        Args:
            project_id (str): Project ID
            descriptor_name (str): Descriptor name. Example: 'my-descriptor'

        Returns:
            result (dict): Metric descriptor
        """
        descriptor = self.client.metric_descriptor(descriptor_name)
        return descriptor
    
    def list_logs_based_metrics(self, project_id):
        """
        List logs-based metrics in a project.

        Args:
            project_id (str): Project ID

        Returns:
            result (list): List of logs-based metrics
        """
        metrics = self.client.list_logs_based_metrics(project_id=project_id)
        return metrics
    
    def get_logs_based_metric(self, project_id, metric_name):
        """
        Get a logs-based metric.

        Args:
            project_id (str): Project ID
            metric_name (str): Metric name. Example: 'my-metric'

        Returns:
            result (dict): Logs-based metric
        """
        metric = self.client.logs_based_metric(metric_name)
        return metric
    
